#!/usr/bin/env python

"""
Prettyprinted text serialiser for Moin nodes.

Copyright (C) 2023 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.serialisers.pretty.common import Serialiser

class MoinSerialiser(Serialiser):

    "Serialisation of Moin nodes for inspection."

    input_formats = ["moin", "wiki"]
    formats = ["pretty"]

    # Node handler methods.

    def region(self, region):
        self.out("%sRegion: level=%d indent=%d type=%s args=%r extra=%r\n" % (
            self.output.indent, region.level, region.indent, region.type, region.args,
            region.extra))
        self.visit_region(region)

    # Block node methods.

    def block(self, block):
        self.out("%sBlock\n" % self.output.indent)
        self.container(block)

    def defitem(self, defitem):
        self.out("%sDefItem: pad=%r extra=%r\n" % (self.output.indent,
            defitem.pad, defitem.extra))
        self.container(defitem)

    def defterm(self, defterm):
        self.out("%sDefTerm: pad=%r extra=%r\n" % (self.output.indent,
            defterm.pad, defterm.extra))
        self.container(defterm)

    def fontstyle(self, fontstyle):
        self.out("%sFontStyle: emphasis=%r strong=%r\n" % (self.output.indent,
            fontstyle.emphasis, fontstyle.strong))
        self.container(fontstyle)

    def heading(self, heading):
        self.out("%sHeading: level=%d start_extra=%r start_pad=%r end_pad=%r"
            " end_extra=%r identifier=%r\n" % (
            self.output.indent, heading.level, heading.start_extra,
            heading.start_pad, heading.end_pad, heading.end_extra,
            heading.identifier))
        self.container(heading)

    def link_label(self, link_label):
        self.out("%sLinkLabel\n" % self.output.indent)
        self.container(link_label)

    def link_parameter(self, link_parameter):
        self.out("%sLinkParameter\n" % self.output.indent)
        self.container(link_parameter)

    def list(self, list):
        self.out("%sList: indent=%r marker=%r num=%r\n" % (
            self.output.indent, list.indent, list.marker, list.num))
        self.container(list)

    def listitem(self, listitem):
        self.out("%sListItem: indent=%d marker=%r space=%r num=%r\n" % (
            self.output.indent, listitem.indent, listitem.marker, listitem.space, listitem.num))
        self.container(listitem)

    def table(self, table):
        self.out("%sTable:\n" % self.output.indent)
        self.container(table)

    def table_attrs(self, table_attrs):
        self.out("%sTableAttrs:\n" % self.output.indent)
        self.container(table_attrs)

    def table_cell(self, table_cell):
        self.out("%sTableCell: leading=%r padding=%r\n" % (
            self.output.indent, table_cell.leading, table_cell.padding))
        self.container(table_cell)

    def table_row(self, table_row):
        self.out("%sTableRow: trailing=%r leading=%r padding=%r\n" % (
            self.output.indent, table_row.trailing, table_row.leading,
            table_row.padding))
        self.container(table_row)

    def inline(self, inline):
        self.out("%s%s\n" % (self.output.indent, inline.__class__.__name__))

    # Inline nodes with children.

    def inline_container(self, inline):
        self.inline(inline)
        self.container(inline)

    larger = inline_container

    def link(self, link):
        self.out("%sLink: target=%r\n" % (self.output.indent, link.target))
        self.container(link)

    def macro(self, macro):
        self.out("%sMacro: name=%r args=%r\n" % (self.output.indent, macro.name, macro.args))
        self.container(macro)

    monospace = inline_container
    smaller = inline_container
    strikethrough = inline_container
    subscript = inline_container
    superscript = inline_container

    def transclusion(self, transclusion):
        self.out("%sTransclusion: target=%r\n" % (self.output.indent, transclusion.target))
        self.container(transclusion)

    underline = inline_container

    # Inline nodes without children.

    def anchor(self, anchor):
        self.out("%sAnchor: target=%r\n" % (self.output.indent, anchor.target))

    break_ = inline

    def comment(self, comment):
        self.out("%sComment: comment=%r extra=%r\n" % (self.output.indent, comment.comment, comment.extra))

    def directive(self, directive):
        self.out("%sDirective: directive=%r extra=%r\n" % (self.output.indent, directive.directive, directive.extra))

    linebreak = inline
    nbsp = inline

    def rule(self, rule):
        self.out("%sRule: height=%d\n" % (self.output.indent, rule.height))

    def table_attr(self, table_attr):
        self.out("%sTableAttr: name=%r value=%r concise=%r quote=%r\n" % (
            self.output.indent, table_attr.name, table_attr.value,
            table_attr.concise, table_attr.quote))

    def text(self, text):
        self.out("%sText: %r\n" % (self.output.indent, text.s))

    def verbatim(self, verbatim):
        self.out("%sVerbatim: text=%r\n" % (self.output.indent, verbatim.text))

serialiser = MoinSerialiser

# vim: tabstop=4 expandtab shiftwidth=4
